/* --------------------------------------------------------------------------------------------------------------------
                                            CURSO DE PROGRAMAO EM PL-SQL
                                                  PEDRO F. CARVALHO
                                              www.pedrofcarvalho.com.br
                                            contato@pedrofcarvalho.com.br
                                              ORACLE 9i, 10G, 11G E 12C
                         Preparao para a prova 1Z0-147 e 1Z0-144 e 1Z0-146 Program with PL/SQL  
----------------------------------------------------------------------------------------------------------------------*/


Captulo 07 - Usando Exceptions.

Na linguagem PL/SQL qualquer tipo de erro  tratado como uma exceo (exception), o que significa uma situao que, a 
princpio, no deveria ocorrer durante a execuo seu programa. Existem dois tipos de exceo:

Exceo de sistema (system exception): um erro definido pelo Oracle que  disparado automaticamente pela runtime engine
da PL/SQL assim que ela detecta o problema. Exemplos: duplicate value in index e no data found.

Exceo Programada: exceo definida pelo programador e, portanto, especfica da aplicao. Este tipo de exceo no  
disparada automaticamente, mas apenas para situaes indicadas pelo programador em seu cdigo.

Neste capitulo, abordaremos o tratamento de excees de programada.

Tratando uma Exceo de programada  Modelo Bsico
O primeiro passo requerido para que voc possa tratar uma exceo programada  inserir um bloco EXCEPTION no final da 
sua procedure ,funo, trigger, bloco anonimo. 

Quando um erro ocorre em qualquer linha do programa PL/SQL, uma exceo  disparada. O processamento do programa 
imediatamente interrompido e o fluxo  desviado para o bloco de excees. Dentro deste bloco, a runtime engine do
PL/SQL tentar localizar o tratador de exceo adequado para o erro ocorrido. 


Um recurso muito comumente utilizado nos programas PL/SQL  utilizar a clusula WHEN OTHERS junto com as funes 
SQLCODE e SQLERRM. Estas funes retornam os seguintes valores:

SQLCODE: retorna o cdigo do ltimo erro ocorrido na execuo do programa.
SQLERRM: retorna a mensagem associada ao erro indicado em SQLCODE.

a Oracle catalogou (ou seja, colocou rtulos) apenas em um conjunto relativamente pequeno de erros, aqueles mais comuns. 
Alguns destes rtulos so apresentados :

invalid cursor - referencia a um cursor invlido
invalid number - erro de tipo de converso de nmeros, nmero invlido
no data found -  select into nao retorna nada, tentativa de acesso a uma linha no existente e UTL_file nao encontrou arquivo
TOO_MANY_ROWS - mais que uma linha

SET SERVEROUTPUT ON
DECLARE    
    V_FN HR.EMPLOYEES.FIRST_NAME%TYPE;  
BEGIN
    SELECT FIRST_NAME INTO V_FN
    FROM   HR.EMPLOYEES
    WHERE  FIRST_NAME = 'David'; 
EXCEPTION
    WHEN TOO_MANY_ROWS THEN    
      DBMS_OUTPUT.PUT_LINE('A consulta retornou mais que uma linha. Considere utilizar colecoes ou cursores.');
END;



SET SERVEROUTPUT ON
DECLARE    
    V_FN HR.EMPLOYEES.FIRST_NAME%TYPE;  
BEGIN
    SELECT FIRST_NAME INTO V_FN
    FROM   HR.EMPLOYEES
    WHERE  FIRST_NAME = 'Davidsdfsdfgsdfgsd'; 
EXCEPTION
    WHEN TOO_MANY_ROWS THEN    
      DBMS_OUTPUT.PUT_LINE('A consulta retornou mais que uma linha. Considere utilizar colecoes ou cursores.');
END;

-- tratamento pre no definido
SET SERVEROUTPUT ON
DECLARE    
    V_FN NUMBER;  
BEGIN
    SELECT FIRST_NAME INTO V_FN
    FROM   HR.EMPLOYEES
    WHERE  ROWNUM = 1; 
EXCEPTION
    WHEN OTHERS THEN    
      DBMS_OUTPUT.PUT_LINE('Ocorreu um erro! ' || SQLERRM);
END;

-- raise aplication error

SET SERVEROUTPUT ON
DECLARE    
    V_FN HR.EMPLOYEES.FIRST_NAME%TYPE;  
    V_COUNT NUMBER;
BEGIN
    SELECT COUNT(1) INTO V_COUNT
    FROM   HR.EMPLOYEES
    WHERE  FIRST_NAME = 'David';  --3
    
    IF V_COUNT > 1 THEN 
          RAISE_APPLICATION_ERROR(-20000, 'Erro customizado. Nao foi possivel gravar valores de multiplas linhas em uma variavel escalar.');
    ELSE
      SELECT FIRST_NAME INTO V_FN
      FROM   HR.EMPLOYEES
      WHERE  FIRST_NAME = 'David';
    END IF;
  
EXCEPTION
    WHEN TOO_MANY_ROWS THEN
          DBMS_OUTPUT.PUT_LINE(SQLERRM);    
END;

 

-- tratamento com bloco filho
SET SERVEROUTPUT ON
DECLARE    
    V_FN HR.EMPLOYEES.FIRST_NAME%TYPE;  
    V_COUNT NUMBER;
BEGIN
    SELECT COUNT(1) INTO V_COUNT
    FROM   HR.EMPLOYEES
    WHERE  FIRST_NAME = 'David'; --3
  
    IF V_COUNT > 0 THEN
        BEGIN -- inicio bloco filho (erro tratado no bloco filho permite continuar execucao do bloco pai (ver linha 23)
            SELECT FIRST_NAME INTO V_FN
            FROM   HR.EMPLOYEES
            WHERE  FIRST_NAME = 'David'; --3
        EXCEPTION
            WHEN TOO_MANY_ROWS THEN    
               DBMS_OUTPUT.PUT_LINE('A consulta retornou mais que uma linha. Considere utilizar collections ou cursores.');
        WHEN OTHERS THEN
            DBMS_OUTPUT.PUT_LINE(SQLERRM);
        END; -- fim do bloco filho
    END IF;

	DBMS_OUTPUT.PUT_LINE('Se ocorrer erro no bloco filho, execucao do bloco pai nao interrompe.');
END;

-- tratamento bloco pai 
SET SERVEROUTPUT ON
DECLARE    
    V_FN HR.EMPLOYEES.FIRST_NAME%TYPE;  
    V_COUNT NUMBER;
BEGIN
    SELECT COUNT(1) INTO V_COUNT
    FROM   HR.EMPLOYEES
    WHERE  FIRST_NAME = 'David';  --3
  
  IF V_COUNT > 0 THEN
      BEGIN -- inicio bloco filho (erro ser propagado p/ o bloco pai
        SELECT FIRST_NAME INTO V_FN
        FROM   HR.EMPLOYEES
        WHERE  FIRST_NAME = 'David';      
      END; -- fim bloco filho
  END IF;
  
  DBMS_OUTPUT.PUT_LINE('Se ocorrer erro no bloco filho, execucao do bloco pai interrompe, pois nao ha tratamento de excecao no bloco filho.');
EXCEPTION
    WHEN TOO_MANY_ROWS THEN    
      DBMS_OUTPUT.PUT_LINE('A consulta retornou mais que uma linha. Considere utilizar collections ou cursores.');
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE(SQLERRM);
END;






