/* --------------------------------------------------------------------------------------------------------------------
                                            CURSO DE PROGRAMAO EM PL-SQL
                                                  PEDRO F. CARVALHO
                                              www.pedrofcarvalho.com.br
                                            contato@pedrofcarvalho.com.br
                                              ORACLE 9i, 10G, 11G E 12C
                         Preparao para a prova 1Z0-147 e 1Z0-144 e 1Z0-146 Program with PL/SQL  
----------------------------------------------------------------------------------------------------------------------*/


Captulo 15 - Manipulating large Objects.

Hoje em dia, cada vez mais, os sistemas esto ficando mais sofisticados e utilizando diversos recursos de desenvolvimento
no banco de dados. Porm, com esse crescimento progressivo das aplicaes, os volumes de dados esto cada vez maiores e 
os bancos precisam de manuteno diria para cuidar desse crescimento. Assim sendo, a Oracle DataBase disponibiliza 
desde a verso 8i, uma tima soluo para armazenagem de informao, ajudando o desenvolvedor e o DBA: o chamado Large Objects.

As Large Objects, ou apelidando carinhosamente de LOBS, so tipos de dados que podem armazenar at 4GB de informao 
binria, como imagens, sons, vdeos, arquivos pdf e etc, ou caracteres, single-byte ou multi-byte, em uma ou mais 
colunas de uma nica tabela do seu banco de dados.

O motivo que leva a pouca utilizao das LOBs,  devido s aplicaes do mercado utilizarem as colunas LONG 
(Para tipo de dados alfanumricos) ou LONG RAW (para tipos de dados binrios) pela alta capacidade
de armazenamento tambm, que pode chegar at 2GB, porm, LONG e LONG RAW no so Lobs!

Isso faz com que o DBAs Oracle, Engenheiros de softwares e desenvolvedores de diversas empresas, perca alguns
cabelos e noites quando temos que falar sobre PERFORMANCE, ARMAZENAMENTO, MANUTENO e COMPARTIBILIDADE
das aplicaes do ambiente. 

Mas, o que existe de to diferente entre LOBs e LONGs?

Bom, para entendermos as diferenas existentes entre elas, devemos analisar as caractersticas de cada
tipo de dados, entender o funcionamento e saber as particularidades de cada um para tirarmos algumas concluses. 
O primeiro a entrar no nosso ring virtual ser o tipo de dados LONG e LONG RAW.

LONG

So tipos de dados alfanumricos com tamanho varivel, seu comprimento mximo pode chegar at 2GB.

LONG RAW

So tipos de dados binrios com tamanho varivel, seu comprimento  igual as LONG, podendo
chegar at 2GB, mas ao declarar um varivel LONG RAW, a especificao do seu tamanho na declarao
 obrigatria.

CARACTERSTICAS

As principais caractersticas que colunas 

LONG e LONG RAW levam no seu currculo, so:

- Tem como restrio
a utilizao de uma coluna por tabela, caso tente
colocar mais colunas, o Oracle no permitir a criao
da tabela, gerando erros;

- O tamanho mximo de armazenamento  2 GB;

- No permite armazenagem de informao fora do banco de dados;

- No suporta tipos de objetos;

- Os comandos SELECT retorna o dado armazenado;

- O acesso aos dados da coluna  feita de forma sequencial, deste modo, a performance da consulta pode
ficar mais pesada;

- No pode ser replicado;

- No participa de transaes com COMMIT ou ROLLBACK;

- Funes SQL e clusulas como WHERE, GROUP BY e ConnectBy no funciona com essas colunas;

- O PL/SQL no interpreta o contudo das colunas LONG RAW.

Agora que sabemos como as LONG trabalham, devemos fazer uma analise por completo das LOBs.

As LOBs podem ser divididas em 4 tipos de dados, como descritos abaixo:

BLOB

So tipos de dados binria.

CLOB

So tipos de dados alfanumricos, podendo armazenar caracteres compatveis com o charset do banco de dados.

NLOB

So tipos de dados alfanumricos, podendo armazenar caracteres compatveis com o national charset do banco de dados.

BFILE

So tipos de dados binrios armazenados fora do banco de dados em arquivos do sistema operacional, com TXT e HTML.

CARACTERSTICAS

- As LOBS podem armazenar at 4GB de dados, isso inclu todos os tipos de LOBs citados acima;

- Podemos trabalhar com 2 modos de operao, INTERNA (Lgica) e EXTERNA (Fsica);

- Permite replicao dos dados, exceto para BFile que trabalha com a parte fsica;

- Suporte a tipos de objetos;

- O sistema de acesso aos dados para as LOBs  de forma randnica, isso permite ganho de performance nas consultas;

- Habilitao para transaes, COMMIT e ROLLBACK, exceto para BFile;

- O comando SELECT retorna os Locators das LOBS e no os dados armazenados;

- Podemos trabalhar com mais de uma coluna LOB nas tabelas, ampliando o armazenamento da tabela;

- Existem pacotes especficos para administrar colunas LOBs no banco de dados;

- Existem views especficas no dicionario de dados para administrao de LOBs dentro do banco de dados;

- Funes SQL podem ser utilizadas, porm, com algumas restries dependendo do tipo de LOB;

Existem ainda muitos recursos especficos das LOBs que devemos comentar e explicar, assim como fazer algumas
comparaes e opinar sobre as vises de um DBA na parte de manuteno e do desenvolvedor no
desenvolvimento com esses tipos de recursos


Qual a diferena entre BLOB x CLOB - Oracle

Um BLOB (binary large object)  um tipo de dados Oracle que pode conter at 4 GB de dados binarios. BLOB so teis para 
armazenar informao digital (por exemplo, imagens, udio, vdeo).

Um CLOB (Character Large objeto)  um tipo de dados Oracle que pode conter at 4 GB de dados. CLOBs so teis 
para armazenar texto.

Tipos de dados BLOB e CLOB so criados atravs da utilizao do CREATE TABLE ou ALTER da mesma forma que so criados 
campos de outros tipos.

-- gravando dados em campo blob

CREATE DIRECTORY IMG AS 'C:\TEMP\';
GRANT READ, WRITE ON DIRECTORY IMG TO HR;

-- exemplo simples de utilizao de campos clob
drop table hr.t_clob
create table hr.t_clob (id number, valor clob);
insert into hr.t_clob values (1, 'texto enorme');
commit;
select * from hr.t_clob
select dbms_lob.substr(valor, 4, 8) from hr.t_clob

-- exemplo simples de utilizao de camps blob
CREATE DIRECTORY IMG AS 'C:\TEMP\';
GRANT READ, WRITE ON DIRECTORY IMG TO HR;

drop table hr.t_blob
create table hr.t_blob (id number, valor blob); --posso ter uma diretiva de segurana e especificar para armazenar em outra tablespace
insert into hr.t_blob values (1, empty_blob());
commit;
select * from hr.t_blob

set serveroutput on
declare
  v_blob blob;
  v_bfile bfile;
begin
  insert into hr.t_blob(id, valor) values (2, empty_blob()) returning valor into v_blob;
  v_bfile:=bfilename('IMG','oracle.jpg');
  dbms_lob.open(v_bfile,dbms_lob.lob_readonly);
  dbms_lob.loadfromfile(v_blob,v_bfile,dbms_lob.getlength(v_bfile));
  dbms_lob.close(v_bfile);
  commit;
end;

select * from hr.t_blob

-- bfile

drop table hr.t_bfile
create table hr.t_bfile (id number, valor bfile);
insert into hr.t_bfile values (1, bfilename('IMG','oracle.jpg'));
commit;

select * from hr.t_bfile

set serveroutput on
declare
  v_bfile bfile;
begin
  select valor into v_bfile from hr.t_bfile where id=1;
  dbms_output.put_line('Arquivo ocupa: '|| dbms_lob.getlength(v_bfile)||' bytes,');
end;

set serveroutput on
declare
  v_bfile bfile;
  v_directorio varchar(255);
  v_archivo varchar(255);
begin
  select valor into v_bfile from hr.t_bfile where id=1;
  dbms_lob.filegetname(v_bfile, v_directorio, v_archivo);
  dbms_output.put_line('Nome do arquivo : '|| v_archivo ||'-'||v_directorio);
end;

-- long
drop table hr.t_long
CREATE TABLE hr.t_long(id NUMBER, doc LONG);
INSERT INTO hr.t_long VALUES (1, 'texto muito longo .....');
COMMIT;
select * from hr.t_long


SecureLobFile
create table hr.t_blob (id number, valor blob lob(valor) store as securefile (tablespace example); -- Oracle 11g
Existe ainda opoes para : deduplicate, compress high medium nocompress

O modo BASICFILE  o nico disponvel at a verso do Oracle 10g.

O Oracle SecureFiles  uma infraestrutura de armazenamento totalmente nova que foi projetada para propiciar melhor 
desempenho e segurana, alm de reduzir o uso de disco com os seus recursos avanados de compresso. 
O desempenho de leitura e escrita tambm  mais alto porque ele oferece uma nova camada de rede cliente/servidor 
permitindo uma transferncia de dados mais veloz.

No Oracle 11g, quando criamos uma coluna do tipo LOB (BLOB/CLOB), por padro o mesmo ser criado no antigo formato 
(BASICFILE). Se quisermos que o padro seja SECUREFILE, deveremos alterar o valor do parmetro db_securefile para 
ALWAYS ou FORCE. Seu valor default  PERMITTED. Vale a pena salientar que esse parmetro pode ser alterado tanto 
no nvel de sistema quanto no nvel de sesso.













 
 
 







