/* --------------------------------------------------------------------------------------------------------------------
                                            CURSO DE PROGRAMAO EM PL-SQL
                                                  PEDRO F. CARVALHO
                                              www.pedrofcarvalho.com.br
                                            contato@pedrofcarvalho.com.br
                                              ORACLE 9i, 10G, 11G E 12C
                         Preparao para a prova 1Z0-147 e 1Z0-144 e 1Z0-146 Program with PL/SQL  
----------------------------------------------------------------------------------------------------------------------*/


Captulo 17 - Segurana contra sql injection - Fine Grained Access.

-- Descrecer SQL Ijnection
-- Use DBMS_Assert
-- Fine Graines Access


A Injeo de SQL, mais conhecida atravs do termo americano SQL Injection,  um tipo de ameaa de segurana que se 
aproveita de falhas em sistemas que interagem com bases de dados via SQL. A injeo de SQL ocorre quando o atacante 
consegue inserir uma srie de instrues SQL dentro de uma consulta (query) atravs da manipulao das entradas de 
dados de uma aplicao

Para exemplificar o funcionamento da injeo de SQL, consideremos uma instruo SQL comum:

SELECT id, nome, sobrenome FROM autores;

Essa instruo, que representa uma consulta na base de dados, retorna todos os registros das colunas "id", "nome" e 
"sobrenome" da tabela "autores". A partir desta mesma instruo, os registros a serem retornados podem ser restritos 
atravs da incluso da clusula WHERE, como  visto no exemplo abaixo:

SELECT id, nome, sobrenome FROM autores WHERE nome = 'jos' AND sobrenome = 'silva';

Com base nesta instruo,  fcil supor que "jos" e "silva" so strings, cujo contedo ser preenchido pela entrada 
feita por algum usurio que estiver fazendo uso da aplicao.

Portanto, supondo que a aplicao no faa o tratamento apropriado do contedo inserido pelo usurio, o mesmo pode 
fazer o uso acidental do caractere de aspas simples. Gerando a entrada:

nome = jo's
sobrenome = silva

E fazendo com que a aplicao gere o cdigo:

SELECT id, nome, sobrenome FROM autores WHERE nome = 'jo's' AND sobrenome = 'silva';

De acordo com a especificao da linguagem SQL, existe um erro de sintaxe nessa instruo, uma vez que a string 
passada para o campo nome  a apenas palavra "jo", pois a adio das aspas simples quebrou a delimitao das aspas 
simples originais da consulta. O interpretador do SQL espera que o restante da instruo seja outros comandos SQL 
vlidos que complementem a instruo principal. No entanto, como "s" no  um identificador vlido, essa instruo 
no ser executada e retornar um erro.


Com base neste problema, um possvel atacante pode manipular os dados de entrada a fim de gerar um comportamento no 
esperado na base de dados.

Para exemplificar este conceito, consideremos na mesma consulta apresentada, a entrada dos seguintes dados pela aplicao:

nome = jo'; DROP TABLE autores ; --
sobrenome = silva

Fazendo com que a aplicao gere o cdigo:

SELECT id, nome, sobrenome FROM autores WHERE nome = 'jo'; DROP TABLE autores ; --' AND sobrenome = 'silva';


Neste caso, a instruo ser executada normalmente, pois no h um erro de sintaxe, no entanto, com a adio do 
caractere ponto-e-vrgula, a instruo foi dada como finalizada de modo prematuro dando espao para uma nova instruo. 
Essa nova instruo, que poderia ser qualquer uma escolhida pelo atacante, pode ser a responsvel por retornar dados 
confidenciais armazenados na base de dados ou de executar instrues que comprometam o sistema, como a remoo de dados
e/ou tabelas, como pode ser visto no exemplo apresentado.

Aparentemente um mtodo para prevenir esse problema seria a remoo de aspas simples dos campos de insero da aplicao,
ou simplesmente no executando a query nestas situaes. Isso  verdade, mas existem vrias dificuldades com esse mtodo 
tanto quanto solues. Primeiro, nem todos os usurios inserem dados em forma de strings. Se o usurio puder selecionar 
um autor pelo 'id' (presumivelmente um nmero) por exemplo, nossa query aparecer como abaixo:


SELECT id, forename, surname FROM authors WHERE id=1234 


Nesta situao, o atacante pode simplesmente adicionar uma instruo SQL no fim do 'input' numrico. Verificando os dialetos de 
SQL, vrios delimitadores podem ser usados no Microsoft Jet DBMS engine, por exemplo, datas podem ser delimitadas com o
caracter sustenido. Portanto, escapando da execuo da adio de aspas simples, no necessariamente uma soluo como 
demonstrado anteriormente.

Pode-se ilustrar esse ponto usando um exemplo de pgina de login em Active Server Pages (ASP), que acessa um servidor 
de banco de dados SQL e tenta autenticar o acesso em uma aplicao fictcia.

Abaixo est um pedao de cdigo de uma pgina de formulrio, em que um usurio insere o username e o password para 
autenticao:

(...)

function Login( cn )
{
   var username;
   var password;
   username = Request.form("username");
   password = Request.form("password");
   var rso = Server.CreateObject("ADODB.Recordset");
   var sql = "select * from users where username = '" + username + "' and password = '" + password + "'";
   trace( "query: " + sql );
   rso.open( sql, cn );
   if (rso.EOF)  {
      rso.close();
   }
}
 
function Main()
{
//Set up connection
   var username
   var cn = Server.createobject( "ADODB.Connection" );
   cn.connectiontimeout = 20;
   cn.open( "localserver", "sa", "password" );
   username = new String( Request.form("username") );
   if( username.length > 0)  {
      Login( cn );
   }
   cn.close();
}


A parte critica  a parte do 'process_login.ascp' que cria uma 'query string':

var sql = "SELECT * FROM users WHERE username = '" + username + "' AND password = '" + password + "'";

Se o usurio inserir os seguintes dados:

var sql = "SELECT * FROM users WHERE username = '" + username: '; drop table users + "' AND password = '" + password + "'";

* Username: '; drop table users--
* Password:
... a tabela 'users' ser apagada, negando o acesso para todos os usurios. 

A seqncia de caracteres '--'  o comentrio de uma linha de SQL, a o ';' denota o fim de uma query e o comeo de 
outra. O '--' no fim do campo username  requerido para que a query em questo seja executada sem erros.

O atacante pode logar como qualquer usurio, se souber o nome do usurio, usando o input abaixo:

Username: admin'-- V
O atacante pode logar como o primeiro usurio da tabelas 'users', com a insero abaixo:

Username: ' or 1=1 -- V
 O atacante pode ainda logar como um usurio completamente fictcio com o input abaixo:

Formans de defesa
   Parametrizao das consultas
   Usar "stored procedures"
   Escapar toda entrada fornecida pelo usurio
   Limitar privilgios aos acessos
   Para se proteger do SQL Injection verifique se todo parmetro passado para o site  tratado antes que seja concatenado na query.
   
   
dbms_assert

https://docs.oracle.com/cd/B28359_01/appdev.111/b28419/d_assert.htm#CHDIFDJE


O pacote DBMS_ASSERT fornece uma interface para validar propriedades do valor de entrada

Adicionar esquerda e  direita aspas simples em uma string literal.
Verifique se todas as aspas simples, exceto esquerda e  direita esto emparelhados com aspas simples adjacentes.


-- retorna com validao
DBMS_ASSERT.ENQUOTE_NAME (
   str            VARCHAR2, 
   capitalize     BOOLEAN DEFAULT TRUE)
RETURN VARCHAR2;

-- retorna sem qualquer validao
DBMS_ASSERT.NOOP (
   str      CLOB CHARACTER SET ANY_CS)
 RETURN     CLOB CHARACTER SET str%CHARSET;
 
 -- verifica se a cadeia de entrara possui um nome sql valido
 DBMS_ASSERT.QUALIFIED_SQL_NAME (
   str      VARCHAR2 CHARACTER SET ANY_CS)
 RETURN     VARCHAR2 CHARACTER SET str%CHARSET;
 
 -- verificar se na cadeia de caractere existe um nome de objeto de eschema
 DBMS_ASSERT.SCHEMA_NAME (
   str      VARCHAR2 CHARACTER SET ANY_CS)
 RETURN     VARCHAR2 CHARACTER SET str%CHARSET;
 
 -- veriica se existe algumc comando sql valido na cadeia de caracteres
 DBMS_ASSERT.SIMPLE_SQL_NAME (
   str      VARCHAR2 CHARACTER SET ANY_CS)
 RETURN     VARCHAR2 CHARACTER SET str%CHARSET;
 
 ORA44002: Invalid object name
 
 --FGA FINE GRANTED ACEESS
 
 o FGA (Fine-grained Audit) espcie de controle de acesso granular. Atravs da DBMS_FGA podemos cadastrar polticas para 
 determinada tabela e ver o resultado na DBA_FGA_AUDIT_TRAIL.

Primeiro vamos entender a DBMS_FGA e o subprograma que adiciona poltica a ADD_POLICY.

DBMS_FGA.ADD_POLICY(
   object_schema   VARCHAR2, 
   object_name     VARCHAR2,    
   policy_name     VARCHAR2, 
   audit_condition VARCHAR2, 
   audit_column    VARCHAR2, 
   handler_schema  VARCHAR2, 
   handler_module  VARCHAR2, 
   enable          BOOLEAN, 
   statement_types VARCHAR2,
   audit_trail     BINARY_INTEGER IN DEFAULT,
   audit_column_opts BINARY_INTEGER IN DEFAULT);

Parmetros

object_schema     => Esquema do objeto.
                     Valor default NULL. Se null ele assume o schema corrente

object_name       => Nome do objeto.

policy_name       => Nome nico da poltica

audit_condition   => Indica uma condicional para a auditoria, exemplo: auditar
                     as queries que possuem, na sua clusula where "sal > 1000". 
                     NULL  o default.

audit_column      => Coluna que ser checada por acesso, NULL  default e causar
                     cheque em todas as colunas acessadas.

handler_schema    => Esquema onde est o event handler. Um event hander, pode ser
                     escrito para tomar alguma deciso quando o audit_condition
                     for verdadeiro.

handler_module    =>  o mdulo (programa) que gerencia a auditoria, geralmente usado
                     como alerta, exemplo, uma procedure que envia e-mail a gerncia
                     alertando sobre uma query contra a tabela de aes, por ai vai 
                     conforme a necessidade do negcio. O handler_module deve possuir
                     a seguinte interface:

                     PROCEDURE  ( 
                          object_schema VARCHAR2, 
                          object_name   VARCHAR2, 
                          policy_name   VARCHAR2 )  AS ...

                     Onde fname  o nome da procedure, object_schema  o nome do
                     esquema, object_name nome da tabela que est auditada e 
                     policy_name nome da poltica.

enable            => true/false - true a poltica estar ativa. Default TRUE.

statement_types   => Onde haver auditoria. Valores aceitos: insert, update,
                     delete ou select. Podemos utilizar todos separados por vrgula.
                     Default: SELECT

audit_trail       => Determina se ser gravado ou no o sql bind e o sql text na 
                     sys.fga_log$. O default  para que seja populado DB_EXTENDED
                     caso contrrio, use DB.


audit_column_opts => Determina se a sentena SQL ser audita quando ela acessa
                     QUALQUER (ANY) ou TODAS (ALL) as colunas. Default ANY_COLUMNS.
                     
                     
EXEMPLO PRATICO

  exec dbms_fga.drop_policy( -
   object_schema => 'sys', -
   object_name   => 'TABEL_FGA', -
   policy_name   => 'EXPENSIVE_BOOKS' -
);
 
begin
   dbms_fga.add_policy(
      object_schema   => 'sys',
      object_name     => 'TABEL_FGA',
      policy_name     => 'LOGS_TABELA_FGA',
      audit_condition => 'salary>=50',
      audit_column    => 'salary',
      handler_schema  => null,
      handler_module  => null,
      enable          => true
   );
end;
/  

desc dba_fga_audit_trail


select
   timestamp     c1,
   db_user       c2,
   os_user       c3,
   object_schema c4,
   object_name   c5,
   policy_name   c6,
   sql_text      c7
from
   dba_fga_audit_trail
order by
   timestamp;
 
 
   
   
Time         DB    OS                    Object     Policy              
Stamp        User  User            Schem Name       name                
------------ ----- --------------- ----- ---------- --------------------
SQL statement                                                           
----------------------------------------                                
20-MAY-03    FRED  BASK\Owner      PUBS  BOOK       EXPENSIVE_BOOKS     
select * from pubs.book                                                 
                                                                        
20-MAY-03    FRED  BASK\Owner     PUBS  BOOK       EXPENSIVE_BOOKS     
select * from pubs.book                                                 
                                                                        
20-MAY-03    FRED  BASK\Owner      PUBS  BOOK       EXPENSIVE_BOOKS      
select book_title, book_type from pubs.book where book_retail_price > 60










