/* --------------------------------------------------------------------------------------------------------------------
                                            CURSO DE PROGRAMAO EM PL-SQL
                                                  PEDRO F. CARVALHO
                                              www.pedrofcarvalho.com.br
                                            contato@pedrofcarvalho.com.br
                                              ORACLE 9i, 10G, 11G E 12C
                         Preparao para a prova 1Z0-147 e 1Z0-144 e 1Z0-146 Program with PL/SQL  
----------------------------------------------------------------------------------------------------------------------*/


Captulo 18 - Meta Dados(DBMS_METADATA) e External Programs.

Para aqueles que ainda no conhecem a package DBMS_METADATA, o mesma est disponvel desde a verso Oracle 9i. 
Esta package  definida durante a criao do banco de dados atravs do script $ORACLE_HOME/rdbms/admin/dbmsmeta.sql. 
Mas, para que serve esta package? Na verdade, esta package disponibiliza uma interface pblica com uma API utilizada 
para a extrao de metadados de objetos de banco de dados. Todas as ferramentas GUI que fornecem suporte  extrao 
de comandos DDLs (Linguagem de definio de dados) de objetos de banco de dados Oracle, acredito eu, fazem uso do pacote DBMS_METADATA. 

Portanto, eu vou alterar o parmetro SQLTERMINATOR para TRUE de modo a adicionar um terminador SQL (; ou /) para cada 
sentena DDL gerada, j que o seu valor padro  FALSE. Outra alterao que realizarei ser a de alterar o parmetro 
SEGMENT_ATTRIBUTES para FALSE, de forma a suprimir informaes de armazenamento de segmentos (storage attributes) dos 
comandos DDLs gerados. Vale a pena salientar que o parmetro STORAGE mesmo estando setado para TRUE, ser ignorado 
quando o parmetro SEGMENT_ATTRIBUTES for setado para FALSE. Sem mais, vamos ento a alguns exemplos prticos:


begin
	-- Acrescenta um terminador SQL (; ou /) para cada sentenca DDL que sera gerada 
	dbms_metadata.set_transform_param(dbms_metadata.session_transform,'SQLTERMINATOR',true);
  
	-- Suprime qualquer informacao de atributos de armazenamento de segmentos
	dbms_metadata.set_transform_param(dbms_metadata.session_transform,'SEGMENT_ATTRIBUTES',false);
end;

-- recuperando DDL da tabela HR.EMPLOYEES
SELECT  DBMS_METADATA.GET_DDL(OBJECT_TYPE => 'TABLE',  
                NAME => 'EMPLOYEES', 
                SCHEMA => 'HR')
FROM    DUAL;

-- recuperando DDL de todos os objetos do schema HR
select  dbms_metadata.GET_DDL(u.object_type,u.object_name,'HR')
from    dba_objects u
where   owner = 'HR'
and     object_type = 'TABLE';


select distinct(object_type)  from dba_objects

-- recuperando DDL das constraints da tabela HR.EMPLOYEES
SELECT  DBMS_METADATA.GET_DEPENDENT_DDL(OBJECT_TYPE => 'CONSTRAINT',  
                BASE_OBJECT_NAME => 'EMPLOYEES', 
                BASE_OBJECT_SCHEMA => 'HR')
FROM    DUAL;


-- recuperando privilegios de sistema do usuario HR
SELECT DBMS_METADATA.GET_GRANTED_DDL(OBJECT_TYPE => 'SYSTEM_GRANT',
                              GRANTEE => 'HR')
FROM DUAL;

-- recuperando privilegios de objeto do usuario HR    
SELECT DBMS_METADATA.GET_GRANTED_DDL(OBJECT_TYPE => 'OBJECT_GRANT',
                         GRANTEE => 'HR')
FROM DUAL;



Procedures Externas ou Programas Externos :

Rotinas externas fornecem a capacidade de comunicao entre a base de dados e programas externos em C, C++, Cobol, 
Fortran, PL1, Visual Basic e Java. Neste assunto estaremos abordando as implantaes de bibliotecas C e Java como
rotinas externas.
Objetivo desses procedimentos externos  que seja capaz de se comunicar atravs de PL/SQL. Usando o Oracle Call Interface (OCI)

https://docs.oracle.com/cd/B12037_01/appdev.101/b10795/adfns_ex.htm



/* This sample shows how to call a PL/SQL stored procedure using the SQL92
 * syntax.  See also the other sample PLSQL.java. 
 */
import java.sql.*;
import java.io.*;

class PLSQLExample
{
  public static void main (String args [])
       throws SQLException, IOException
  {
    // Load the driver
    DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());

    // Connect to the database
    // You can put a database name after the @ sign in the connection URL.
    Connection conn =
      DriverManager.getConnection ("jdbc:oracle:oci8:@", "scott", "tiger");

    // Create a statement
    Statement stmt = conn.createStatement ();

    // Create the stored function
    stmt.execute ("create or replace function RAISESAL (name CHAR, raise NUMBER)
                 return NUMBER is begin return raise + 100000; end;");

    // Close the statement
    stmt.close();

    // Prepare to call the stored procedure RAISESAL.
    // This sample uses the SQL92 syntax
    CallableStatement cstmt = conn.prepareCall ("{? = call RAISESAL (?, ?)}");

    // Declare that the first ? is a return value of type Int
    cstmt.registerOutParameter (1, Types.INTEGER);

    // We want to raise LESLIEs salary by 20,000
    cstmt.setString (2, "LESLIE");  // The name argument is the second ?
    cstmt.setInt (3, 20000);        // The raise argument is the third ?
  
    // Do the raise
    cstmt.execute ();

    // Get the new salary back
    int new_salary = cstmt.getInt (1);

    System.out.println ("The new salary is: " + new_salary);

    // Close the statement
    cstmt.close();

    // Close the connection
    conn.close();
  }
}















